/*
Copyright Rene Rivera 2008-2013
Distributed under the Boost Software License, Version 1.0.
(See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)
*/

#ifndef MSGPACK_PREDEF_LIBRARY_C__PREFIX_H
#define MSGPACK_PREDEF_LIBRARY_C__PREFIX_H

#include <msgpack/predef/detail/_cassert.h>

#endif
