//
// MessagePack for C++ zero-copy buffer implementation
//
// Copyright (C) 2018 KONDO Takatoshi
//
//    Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//    http://www.boost.org/LICENSE_1_0.txt)
//
#ifndef MSGPACK_V3_VREFBUFFER_DECL_HPP
#define MSGPACK_V3_VREFBUFFER_DECL_HPP

#include "msgpack/v2/vrefbuffer_decl.hpp"

namespace msgpack {

/// @cond
MSGPACK_API_VERSION_NAMESPACE(v3) {
/// @endcond

using v2::vrefbuffer;

/// @cond
}  // MSGPACK_API_VERSION_NAMESPACE(v3)
/// @endcond

}  // namespace msgpack

#endif // MSGPACK_V3_VREFBUFFER_DECL_HPP
